import pickle
from googleapiclient.discovery import build

import os
from dotenv import load_dotenv
load_dotenv()

with open(os.getenv("GOOGLE_CALENDAR_PICKLE"), 'rb') as f:
    creds = pickle.load(f)

class Calendar():

    def __init__(self, calendarId):

        self.service = build(
            "calendar", "v3",
            credentials=creds,
            cache_discovery=False
        )
        self.calendarId = calendarId

    def list_events(self, start_date):

        """
        :param start_date: The date filter for the events. Expects an ISO formatted date object
        Example          : pendulum.datetime(2020, 9, 1).isoformat()

        :return: Dictionary of event items
        """

        page_token = None
        events = []
        count = 0

        while True:
            # Remember that maxResults does not guarantee the number of results on one page
            # Use pagination instead: https://developers.google.com/calendar/v3/pagination
            # Code for generating page tokens: https://developers.google.com/calendar/v3/reference/calendarList/list#python
            events_list = self.service.events().list(
                calendarId=self.calendarId,
                pageToken=page_token,
                timeMin=start_date,
                maxResults=100,
                singleEvents=True,
                orderBy='startTime'
            ).execute()

            for event in events_list['items']:
                events.append(event)

            page_token = events_list.get('nextPageToken')

            count += 1

            print(f"Google Calendar: Fetching results from page {count} ...")

            if not page_token:
                break

        print(f"Fetched a total of {len(events)} events.")
        return events


__version__ = '0.20.6'

from data_connectors.slack import Slack
from data_connectors.drive import Drive
from data_connectors.gsheets import Gsheets
from data_connectors.mssql import SQLServer
from data_connectors.bigquery import BigQuery
from data_connectors.calendar import Calendar
from data_connectors.transform import Transform


import os
import pygsheets
from dotenv import load_dotenv
load_dotenv()

class Gsheets:
    """
    Pass in a service account file path and query data stored in Google Sheets
    Example: Gsheets().query('SPREADSHEET', 'WORKSHEET')
    """
    def __init__(self):
        self.client = pygsheets.authorize(service_file=(os.getenv("GSHEETS_SERVICE_ACCOUNT")))

    def query(self, spreadsheet, worksheet):
        """
        Returns results of spreadsheet, worksheet as a pandas DataFrame
        Spreadsheet is stored as env var
        Worksheet is passed in as plain text
        """        
        return self.client.open_by_key(os.getenv(spreadsheet)).worksheet_by_title(worksheet).get_as_df()


import pendulum
import pandas as pd


class Transform:
    
    def __init__(self, df):
        self.df = df

    def lower_case_columns(self):
        self.df.columns = self.df.columns.str.lower().str.replace(' ', '_')
        return self

    def add_updated_timestamp(self):
        # Add a UTC timestamp
        self.df['updated_at'] = pendulum.now().strftime('%Y-%m-%d %H:%M:%S %p')
        self.df['updated_at'] = self.df['updated_at'].apply(pd.to_datetime, format='%Y-%m-%d %H:%M:%S %p', utc=True)
        return self


import os
from dotenv import load_dotenv
from sqlalchemy import create_engine
load_dotenv()

class SQLServer:
    """
    Pass in a conn string to generate the engine for querying
    Example: pd.read_sql(query, SQLServer(SERVER="EXAMPLE_SERVER_CONN_STRING").engine)
    
    SQLAlchemy EXAMPLE_SERVER_CONN_STRING Format:
    mssql+pyodbc://user:password@server-ip/database?driver=ODBC+Driver
    """
    def __init__(self, SQLSERVER_CONN_STR):
        self.engine = create_engine(os.getenv(SQLSERVER_CONN_STR))